@@ -11,7 +11,9 @@ from pygradflow.timer import SimpleTimer
 
 
 class StateData:
-    def __init__(self):
+    def __init__(self, iterate, step_result):
+        self.iterate = iterate
+        self.step_result = step_result
         self._entries = dict()
 
     def __setitem__(self, key, value):
@@ -21,7 +23,7 @@ class StateData:
         try:
             entry = self._entries[key]
             if callable(entry):
-                return entry()
+                return entry(self.iterate, self.step_result)
             return entry
         except Exception:
             return None
@@ -116,7 +118,7 @@ class Display:
 
         self.timer = None
         if self.interval is not None:
-            assert self.interval > 0
+            assert self.interval >= 0
             self.timer = SimpleTimer()
         self.last_state = None
 
@@ -219,7 +221,12 @@ def solver_display(problem: Problem, params: Params):
     cols.append(
         AttrColumn("Primal step", 16, "{:16.8e}", StateAttr("primal_step_norm"))
     )
-    cols.append(AttrColumn("Dual step", 16, "{:16.8e}", StateAttr("dual_step_norm")))
+
+    if problem.num_cons > 0:
+        cols.append(
+            AttrColumn("Dual step", 16, "{:16.8e}", StateAttr("dual_step_norm"))
+        )
+
     cols.append(AttrColumn("Lambda", 16, "{:16.8e}", StateAttr("lamb")))
 
     if problem.var_bounded:
@@ -328,7 +335,9 @@ def print_problem_stats(problem, iterate):
     bounded_below = np.logical_and(np.logical_not(inf_lb), inf_ub)
     bounded_above = np.logical_and(inf_lb, np.logical_not(inf_ub))
     bounded = np.logical_or(bounded_below, bounded_above)
+    fixed = var_lb[ranged] == var_ub[ranged]
 
+    num_fixed = np.sum(fixed)
     num_unbounded = np.sum(unbounded)
     num_ranged = np.sum(ranged)
     num_bounded = np.sum(bounded)
@@ -339,6 +348,7 @@ def print_problem_stats(problem, iterate):
     if has_cons:
         logger.info("          Jacobian nnz: %s", cons_jac.nnz)
 
+    logger.info("       Fixed variables: %s", num_fixed)
     logger.info("        Free variables: %s", num_unbounded)
     logger.info("     Bounded variables: %s", num_bounded)
     logger.info("      Ranged variables: %s", num_ranged)
