@@ -194,6 +194,11 @@ class IntegrationSolver:
 
                 assert not (filter[j])
 
+                # FIXME: These conditions may not be satisfied if rhs has large
+                # derivative (guarantee in terms of quality of the root is
+                # in terms of primal distance, not function residuum)
+                #
+                # Do we need to refine the solution??
                 if at_lb:
                     assert func_pos(rhs, rhs_deriv, j)
                 elif at_ub:
@@ -352,7 +357,7 @@ class IntegrationSolver:
 
     def solve(self, x0: Optional[np.ndarray] = None, y0: Optional[np.ndarray] = None):
 
-        self.transform = Transformation(self.orig_problem, self.params, x0, y0)
+        self.transform = Transformation(self.orig_problem, self.params)
 
         self.problem = self.transform.trans_problem
         self.evaluator = self.transform.evaluator
@@ -362,7 +367,7 @@ class IntegrationSolver:
         self.flow = Flow(problem, params, self.evaluator)
         self.rho = self.params.rho
 
-        initial_iterate = self.transform.initial_iterate
+        initial_iterate = self.transform.create_transformed_iterate(x0, y0)
 
         self.path = [initial_iterate.z[:, None]]
         self.path_times = [np.array([0.0])]
@@ -432,7 +437,7 @@ class IntegrationSolver:
             path_dist += result.dist
 
             if display.should_display():
-                state = StateData()
+                state = StateData(curr_it, step_result=None)
                 curr_x, curr_y = self.flow.split_states(curr_z)
                 iterate = curr_it
                 state["iterate"] = iterate
