@@ -1,4 +1,4 @@
-from typing import List, Optional
+from typing import List, Optional, Tuple
 
 import numpy as np
 
@@ -7,7 +7,7 @@ from pygradflow.display import Format, StateData, print_problem_stats, solver_di
 from pygradflow.eval import EvalError
 from pygradflow.iterate import Iterate
 from pygradflow.log import logger
-from pygradflow.params import Params
+from pygradflow.params import Params, PenaltyUpdate
 from pygradflow.penalty import penalty_strategy
 from pygradflow.problem import Problem
 from pygradflow.result import SolverResult
@@ -84,17 +84,21 @@ class Solver:
         self.params = params
         self.callbacks = Callbacks()
 
-    def compute_step(
+        self.transform = Transformation(self.orig_problem, self.params)
+        self.problem = self.transform.trans_problem
+
+    def _compute_step(
         self,
         controller: StepController,
         iterate: Iterate,
+        rho: float,
         dt: float,
         display: bool,
         timer: Timer,
     ) -> StepControlResult:
 
-        assert self.rho != -1.0
-        return controller.compute_step(iterate, self.rho, dt, display, timer)
+        assert rho != -1.0
+        return controller.compute_step(iterate, rho, dt, display, timer)
 
     def _deriv_check(self, x: np.ndarray, y: np.ndarray) -> None:
         from pygradflow.deriv_check import deriv_check
@@ -134,6 +138,9 @@ class Solver:
         iterations: int,
         accepted_steps: int,
         dist_factor: float,
+        rho_init: float,
+        rho_final: float,
+        num_penalty_changes: float,
     ) -> None:
         rho = self.rho
 
@@ -149,6 +156,11 @@ class Solver:
 
         logger.info("%20s: %45e", "Distance factor", dist_factor)
 
+        if self.params.penalty_update != PenaltyUpdate.Constant:
+            logger.info("%20s: %45e", "Initial penalty", rho_init)
+            logger.info("%20s: %45e", "Final penalty", rho_final)
+            logger.info("%20s: %45d", "Penalty changes", num_penalty_changes)
+
         logger.info("%20s: %45e", "Objective", iterate.obj)
         logger.info("%20s: %45e", "Aug Lag violation", iterate.aug_lag_violation(rho))
         logger.info("%20s: %45e", "Aug Lag dual", iterate.aug_lag_dual())
@@ -192,6 +204,32 @@ class Solver:
             logger.debug("Unboundedness detected")
             return SolverStatus.Unbounded
 
+    def perform_iteration(
+        self, x0: np.ndarray | float | None = None, y0: np.ndarray | float | None = None
+    ) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+        params = self.params
+        problem = self.problem
+        iterate = self.transform.create_transformed_iterate(x0, y0)
+
+        lamb = params.lamb_init
+        rho_init = params.rho
+
+        controller = step_controller(problem, params)
+
+        timer = Timer(float(np.inf))
+
+        step_result = self._compute_step(
+            controller, iterate, rho_init, 1.0 / lamb, False, timer
+        )
+
+        iterate = step_result.iterate
+
+        x = iterate.x
+        y = iterate.y
+        d = iterate.bounds_dual
+
+        return self.transform.restore_sol(x, y, d)
+
     def solve(
         self,
         x0: np.ndarray | float | None = None,
@@ -214,21 +252,17 @@ class Solver:
             solutions
         """
 
-        self.transform = Transformation(self.orig_problem, self.params, x0, y0)
-
-        self.problem = self.transform.trans_problem
-
         params = self.params
         problem = self.problem
 
         self.evaluator = self.transform.evaluator
 
-        self.penalty = penalty_strategy(self.problem, params)
+        self.penalty_strategy = penalty_strategy(self.problem, params)
         self.rho = -1.0
 
         display = solver_display(problem, params)
 
-        iterate = self.transform.initial_iterate
+        iterate = self.transform.create_transformed_iterate(x0, y0)
 
         try:
             iterate.check_eval()
@@ -243,7 +277,8 @@ class Solver:
 
         self._deriv_check(iterate.x, iterate.y)
 
-        self.rho = self.penalty.initial(iterate)
+        rho_init = self.penalty_strategy.initial(iterate)
+        self.rho = rho_init
 
         logger.debug("Initial Aug Lag: %.10e", iterate.aug_lag(self.rho))
 
@@ -257,6 +292,8 @@ class Solver:
         accepted_steps = 0
         iteration = 0
 
+        num_penalty_changes = 0
+
         timer = Timer(params.time_limit)
 
         if params.collect_path:
@@ -272,8 +309,8 @@ class Solver:
 
             display_iterate = display.should_display()
 
-            step_result = self.compute_step(
-                controller, iterate, 1.0 / lamb, display_iterate, timer
+            step_result = self._compute_step(
+                controller, iterate, self.rho, 1.0 / lamb, display_iterate, timer
             )
 
             x = iterate.x
@@ -294,29 +331,31 @@ class Solver:
             self.callbacks(CallbackType.ComputedStep, iterate, next_iterate, accept)
 
             if display_iterate:
-                state = StateData()
+                state = StateData(iterate, step_result)
                 state["iterate"] = iterate
-                state["active_set"] = lambda: step_result.active_set
-                state["obj_nonlin"] = lambda: iterate.obj_nonlin(next_iterate)
+                state["active_set"] = lambda _, step_result: step_result.active_set
+                state["obj_nonlin"] = lambda iterate, _: iterate.obj_nonlin(
+                    step_result.iterate
+                )
 
                 if problem.num_cons > 0:
-                    state["cons_nonlin"] = lambda: np.linalg.norm(
-                        iterate.cons_nonlin(next_iterate), ord=np.inf
+                    state["cons_nonlin"] = lambda iterate, _: np.linalg.norm(
+                        iterate.cons_nonlin(step_result.iterate), ord=np.inf
                     )
 
-                state["aug_lag"] = lambda: iterate.aug_lag(self.rho)
-                state["obj"] = lambda: iterate.obj()
+                state["aug_lag"] = lambda iterate, _: iterate.aug_lag(self.rho)
+                state["obj"] = lambda iterate, _: iterate.obj()
                 state["iter"] = iteration + 1
-                state["primal_step_norm"] = lambda: primal_step_norm
-                state["dual_step_norm"] = lambda: dual_step_norm
-                state["lamb"] = lambda: lamb
-                state["step_accept"] = lambda: accept
-                state["rcond"] = lambda: step_result.rcond
+                state["primal_step_norm"] = primal_step_norm
+                state["dual_step_norm"] = dual_step_norm
+                state["lamb"] = lamb
+                state["step_accept"] = accept
+                state["rcond"] = lambda _, step_result: step_result.rcond
 
                 logger.info(display.row(state))
 
             if accept:
-                penalty_result = self.penalty.update(iterate, next_iterate)
+                penalty_result = self.penalty_strategy.update(iterate, next_iterate)
                 next_rho = penalty_result.next_rho
                 accept = penalty_result.accept
 
@@ -327,8 +366,7 @@ class Solver:
                         "Updating penalty parameter from %e to %e", self.rho, next_rho
                     )
                     self.rho = next_rho
-
-                delta = iterate.dist(next_iterate)
+                    num_penalty_changes += 1
 
                 if path is not None:
                     path.append(next_iterate.z)
@@ -339,11 +377,6 @@ class Solver:
                 path_dist += primal_step_norm + dual_step_norm
                 accepted_steps += 1
 
-                if (lamb <= params.lamb_term) and (delta <= params.opt_tol):
-                    logger.debug("Convergence achieved")
-                    status = SolverStatus.Optimal
-                    break
-
             iteration += 1
 
         total_time = timer.elapsed()
@@ -364,6 +397,9 @@ class Solver:
             iterations=iterations,
             accepted_steps=accepted_steps,
             dist_factor=dist_factor,
+            rho_init=rho_init,
+            rho_final=self.rho,
+            num_penalty_changes=num_penalty_changes,
         )
 
         x = iterate.x
